const { describe, it, beforeEach, afterEach } = require("mocha");
const { transferSulu } = require("./transferSulu");
const { transferPlugin } = require("./transferPlugins");
const assert = require("assert");
const fs = require("fs-extra");
const path = require("path");

const {
  macros,
  beforeEachTest,
  afterEachTest
} = require("../../resources/tests");

const {
  testSuluDir,
  testWriteStage1ResultDir,
  log,
  mockPluginManifest,
  testPluginsDirsList,
  mockPluginMatchContents,
} = macros();

describe("src/libs/stage1/transferPlugins.spec.js", () => {
  let rootfiles, srcfiles;
  beforeEach(() => {
    beforeEachTest();
    transferSulu({ 
      sourcePath: testSuluDir,
      buildOutputPath: testWriteStage1ResultDir,
      log,
      incrementalBuild: false,
      buildManifest: {} 
    });
    transferPlugin({
      manifest: {
        absolutePath: testPluginsDirsList[0],
        manifest: mockPluginManifest
      },
      buildOutputPath: testWriteStage1ResultDir,
      allowPluginsToOverride: false,
      log,
      buildManifest: {} 
    });
    rootfiles = fs.readdirSync(testWriteStage1ResultDir);
    srcfiles = fs.readdirSync(path.join(testWriteStage1ResultDir, "src"));
  });

  afterEach(() => {
    afterEachTest();
  });

  describe("transferPlugin()", () => {
    it("should not transfer the file that does not match the pattern", () => {
      assert.equal(srcfiles.includes("noPatternMatch.c"), false);
    });
    it("should include package.json", () => {
      assert.equal(rootfiles.includes("package.json"), true);
    });
    it("should transfer file that matches the pattern", () => {
      assert.equal(srcfiles.includes("includePatternMatch.js"), true);
    });
    it("should not transfer file that matches the pattern and matches exclude pattern", () => {
      assert.equal(srcfiles.includes("includeExcludePatternMatch.js"), false);
    });
    it("should not transfer anything out of node_modules", () => {
      assert.equal(srcfiles.includes("node_modules"), false);
    });
    it("should correctly override a file with second plugin", () => {
      const beforeOverride = fs.readFileSync(path.join(testWriteStage1ResultDir, "src", "includePatternMatch.js")).toString();
      transferPlugin({
        manifest: {
          absolutePath: testPluginsDirsList[1],
          manifest: mockPluginManifest
        },
        buildOutputPath: testWriteStage1ResultDir,
        allowPluginsToOverride: true,
        log,
        buildManifest: {}
      });
      rootfiles = fs.readdirSync(testWriteStage1ResultDir);
      srcfiles = fs.readdirSync(path.join(testWriteStage1ResultDir, "src"));
      const afterOverride = fs.readFileSync(path.join(testWriteStage1ResultDir, "src", "includePatternMatch.js")).toString();
      assert.deepEqual(beforeOverride, mockPluginMatchContents[0]);
      assert.deepEqual(afterOverride, mockPluginMatchContents[1]);
    });
    it("should transfer files from cypress folder (so not just root)", () => {
      assert.equal(rootfiles.includes("cypress"), true);
      const cyprFiles = fs.readdirSync(path.join(testWriteStage1ResultDir, "cypress"));
      assert.deepStrictEqual(cyprFiles, [ "index.spec.js", "plugin-0.spec.js" ]);
    });
  });
});
